package tv.acfun.cloud.service.limit.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import tv.acfun.cloud.common.entity.TargetType;
import tv.acfun.cloud.common.mongo.SequenceRepository;
import tv.acfun.cloud.common.resolver.JsonArgument;
import tv.acfun.cloud.common.util.BeanSolver;
import tv.acfun.cloud.service.limit.LimitConstants;
import tv.acfun.cloud.service.limit.LimitService;
import tv.acfun.cloud.service.limit.entity.LimitEntity;
import tv.acfun.cloud.service.limit.entity.TargetEntity;
import tv.acfun.cloud.service.limit.repository.LimitRepository;
import tv.acfun.cloud.service.limit.repository.TargetRepository;
import tv.acfun.cloud.service.user.entity.RoleEntity;
import tv.acfun.cloud.service.user.entity.UserEntity;
import tv.acfun.cloud.service.user.entity.UserRoleEntity;

import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created by wzp on 16/02/02.
 */
@RestController
@CacheConfig(cacheNames = LimitConstants.LIMITS_CACHE)
public class LimitController implements LimitService {

  @Autowired
  private SequenceRepository sequenceRepository;
  @Autowired
  private LimitRepository limitRepository;
  @Autowired
  private TargetRepository targetRepository;

  @Transactional(readOnly = true)
  @Cacheable
  public List<LimitEntity> findByV10(@JsonArgument UserEntity user, @RequestParam Long typeId) {
    if (user == null || typeId == -1) {
      List<LimitEntity> adminResult = limitRepository.findByDeletedAtNull();
      adminResult.stream().filter(e -> e.getExpiredAt() == null || e.getExpiredAt().after(new Date())).collect(Collectors.toList());
      return adminResult;
    }
    //获得userId和相应地roleId列表
    Long userId = null;
    Set<RoleEntity> setRoles = new HashSet<>();
    if (user != null) {
      userId = Long.parseLong(user.getId().toString());
      Set<UserRoleEntity> userRoleEntities = user.getUserRoles();
      if (userRoleEntities != null) {
        setRoles = userRoleEntities.stream().map(userRoleEntity -> userRoleEntity.getRole()).collect(Collectors.toSet());
      }
    }
    Set<Long> roleIds = new HashSet<>();
    setRoles.stream().forEach(e -> roleIds.add(Long.parseLong(e.getId().toString())));

    //获得所有该用户相关的封禁目标列表
    List<TargetEntity> tmpAllTargets = targetRepository.findByType(TargetType.ALL.getId());
    List<TargetEntity> allTargets = tmpAllTargets == null ? new ArrayList<>() : tmpAllTargets;
    List<TargetEntity> userTargets = new ArrayList<TargetEntity>();
    if (userId != null) {
      userTargets = targetRepository.findByTypeAndTargetId(TargetType.USER.getId(), userId);
    }
    List<TargetEntity> roleTargets = new ArrayList<TargetEntity>();
    Long[] rleIds = (Long[]) roleIds.toArray(new Long[roleIds.size()]);
    List<TargetEntity> tempTargets = targetRepository.findByTypeAndTargetIdIn(TargetType.ROLE.getId(), rleIds);
    roleTargets.addAll(tmpAllTargets);

    //获得该用户所有的limitId
    Set<Long> limitIds = new HashSet<>();
    limitIds.addAll(allTargets.stream().map(e -> e.getLimitId()).collect(Collectors.toList()));
    limitIds.addAll(userTargets.stream().map(e -> e.getLimitId()).collect(Collectors.toList()));
    limitIds.addAll(roleTargets.stream().map(e -> e.getLimitId()).collect(Collectors.toList()));

    //根据limitIds查询所有的封禁信息
    List<LimitEntity> result = new ArrayList<LimitEntity>();
    Long[] lmtIds = (Long[]) limitIds.toArray(new Long[limitIds.size()]);
    result.addAll(limitRepository.findByIdInAndDeletedAtNull(lmtIds));

    //去除expireAt大于当前时间和为null的封禁结果集
    result = result.stream().filter(e -> e.getExpiredAt() == null || e.getExpiredAt().after(new Date())).collect(Collectors.toList());

    //根据封禁类型过滤结果集
    result = result.stream().filter(
        e -> e.getTypes().stream().map(t -> t.getId()).collect(Collectors.toList())
            .contains(typeId)
    ).collect(Collectors.toList());

    return result;
  }

  @Transactional(readOnly = true)
  @Cacheable
  public List<LimitEntity> findByOnlyUserV10(@JsonArgument UserEntity user, @RequestParam Long typeId) {

    //获得userId和相应地roleId列表
    Long userId = null;
    Set<RoleEntity> setRoles = new HashSet<>();
    if (user != null) {
      userId = Long.parseLong(user.getId().toString());
      Set<UserRoleEntity> userRoleEntities = user.getUserRoles();
      if (userRoleEntities != null) {
        setRoles = userRoleEntities.stream().map(userRoleEntity -> userRoleEntity.getRole()).collect(Collectors.toSet());
      }
    }
    Set<Long> roleIds = new HashSet<>();
    setRoles.stream().forEach(e -> roleIds.add(Long.parseLong(e.getId().toString())));

    //获得该用户(不包括TargetType.ALL与TargetType.ROLE)相关的封禁目标列表
    List<TargetEntity> userTargets = new ArrayList<TargetEntity>();
    if (userId != null) {
      userTargets = targetRepository.findByTypeAndTargetId(TargetType.USER.getId(), userId);
    }

    //获得该用户所有的limitId
    Set<Long> limitIds = new HashSet<>();
    limitIds.addAll(userTargets.stream().map(e -> e.getLimitId()).collect(Collectors.toList()));

    //根据limitIds查询所有的封禁信息
    List<LimitEntity> result = new ArrayList<LimitEntity>();
    Long[] lmtIds = (Long[]) limitIds.toArray(new Long[limitIds.size()]);
    result.addAll(limitRepository.findByIdInAndDeletedAtNull(lmtIds));

    //过滤出startedAt<当前时间<expireAt的封禁结果集，（expireAt为null代表无限大）
    Date now = new Date();
    result = result.stream().filter(e -> (e.getExpiredAt() == null || e.getExpiredAt().after(now)) && (e.getStartedAt().before(now))).collect(Collectors.toList());

    //根据封禁类型过滤结果集
    result = result.stream().filter(
        e -> e.getTypes().stream().map(t -> t.getId()).collect(Collectors.toList())
            .contains(typeId)
    ).collect(Collectors.toList());

    return result;
  }

  @Transactional(readOnly = true)
  @Cacheable
  public LimitEntity findByIdV10(@PathVariable Long id) {
    LimitEntity result = limitRepository.findByIdAndDeletedAtNull(id);
    if (result.getExpiredAt() != null && result.getExpiredAt().before(new Date())) {
      result = null;
    }
    return result;
  }

  @Transactional
  @CacheEvict(allEntries = true)
  public LimitEntity deleteByIdV10(@PathVariable Long id, @JsonArgument LimitEntity limit) {
    LimitEntity oldLimit = this.findByIdV10(id);
    BeanSolver.copyProperties(limit, oldLimit);
    //删除关联的目标列表
    List<TargetEntity> targets = oldLimit.getTargets();
    if (targets != null && targets.size() > 0) {
      targetRepository.delete(targets);
    }
    //更新封禁对象：设置deleterId和deletedAt
    oldLimit.setId(id);
    return limitRepository.save(oldLimit);
  }

  @Transactional
  @CacheEvict(allEntries = true)
  public LimitEntity saveV10(@JsonArgument LimitEntity limit) {
    //获得limitId
    Long limitId = sequenceRepository.getNextValue(LimitConstants.LIMITS_ID_KEY);
    //保存封禁的目标
    List<TargetEntity> targets = limit.getTargets();
    targets.stream().forEach(e -> {
      Long targetId = sequenceRepository.getNextValue(LimitConstants.TARGETS_ID_KEY);
      e.setId(targetId);
      e.setLimitId(limitId);
    });
    targetRepository.save(targets);
    //保存封禁主体
    limit.setId(limitId);
    return limitRepository.save(limit);
  }

  @Transactional
  @CacheEvict(allEntries = true)
  public LimitEntity updateByIdV10(@PathVariable Long id, @JsonArgument LimitEntity limit) {
    LimitEntity oldLimit = this.findByIdV10(id);
    //保存封禁的目标
    if (limit.getTargets() != null) {
      List<TargetEntity> oldTargets = oldLimit.getTargets();
      List<Long> oldIds = new ArrayList<Long>();
      if (oldTargets != null && oldTargets.size() > 0) {
        oldTargets.stream().forEach(e -> oldIds.add(e.getId()));
      }

      List<TargetEntity> newTargets = limit.getTargets();
      List<Long> newIds = new ArrayList<Long>();
      if (newTargets != null && newTargets.size() > 0) {
        newTargets.stream().forEach(e -> newIds.add(e.getId()));
      }

      for (TargetEntity entity : newTargets) {
        //add
        if (entity.getId() == null) {
          Long targetId = sequenceRepository.getNextValue(LimitConstants.TARGETS_ID_KEY);
          entity.setId(targetId);
          entity.setLimitId(id);
          targetRepository.save(entity);
        } else if (oldIds.contains(entity.getId())) {
          //update
          targetRepository.save(entity);
        }
      }
      //delete
      for (Long oldId : oldIds) {
        if (!newIds.contains(oldId)) {
          targetRepository.delete(oldId);
        }
      }
    }
    //保存封禁主体
    limit.setId(id);
    BeanSolver.copyProperties(limit, oldLimit);
    limitRepository.save(oldLimit);
    return this.findByIdV10(oldLimit.getId());
  }
}
