package tv.acfun.cloud.service.limit.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RestController;
import tv.acfun.cloud.common.entity.Paginate;
import tv.acfun.cloud.common.mongo.SequenceRepository;
import tv.acfun.cloud.common.resolver.JsonArgument;
import tv.acfun.cloud.common.util.BeanSolver;
import tv.acfun.cloud.service.limit.LimitConstants;
import tv.acfun.cloud.service.limit.ReasonService;
import tv.acfun.cloud.service.limit.entity.ReasonEntity;
import tv.acfun.cloud.service.limit.repository.ReasonRepository;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by wzp on 16/02/02.
 */
@RestController
@CacheConfig(cacheNames = LimitConstants.REASONS_CACHE)
public class ReasonController implements ReasonService {

  @Autowired
  private SequenceRepository sequenceRepository;
  @Autowired
  private ReasonRepository reasonRepository;

  @Transactional(readOnly = true)
  @Cacheable
  public Paginate findByV10(@JsonArgument ReasonEntity reason, @JsonArgument Paginate paginate) {
    if (paginate == null) {
      paginate = new Paginate();
    }
    //获得数据结果
    List<ReasonEntity> result = new ArrayList<>();
    if (reason == null || reason.getCreaterId() == null) {
      result = reasonRepository.findByDeletedAtNull(paginate.getPageRequest());
    } else {
      result = reasonRepository.findByCreaterIdAndDeletedAtNull(reason.getCreaterId(), paginate.getPageRequest());
    }
    return new Paginate(result);
  }

  @Transactional(readOnly = true)
  @Cacheable
  public ReasonEntity findByIdV10(@PathVariable Long id) {
    return reasonRepository.findByIdAndDeletedAtNull(id);
  }

  @Transactional
  @CacheEvict(allEntries = true, value = {LimitConstants.REASONS_CACHE, LimitConstants.LIMITS_CACHE})
  public ReasonEntity deleteByIdV10(@PathVariable Long id, @JsonArgument ReasonEntity reason) {
    ReasonEntity oldReason = this.findByIdV10(id);
    //更新类型对象：设置deleterId和deletedAt
    reason.setId(id);
    BeanSolver.copyProperties(reason, oldReason);
    return reasonRepository.save(oldReason);
  }

  @Transactional
  @CacheEvict(allEntries = true, value = {LimitConstants.REASONS_CACHE, LimitConstants.LIMITS_CACHE})
  public ReasonEntity saveV10(@JsonArgument ReasonEntity reason) {
    //保存原因主体
    Long reasonId = sequenceRepository.getNextValue(LimitConstants.REASONS_ID_KEY);
    reason.setId(reasonId);
    return reasonRepository.save(reason);
  }

  @Transactional
  @CacheEvict(allEntries = true, value = {LimitConstants.REASONS_CACHE, LimitConstants.LIMITS_CACHE})
  public ReasonEntity updateByIdV10(@PathVariable Long id, @JsonArgument ReasonEntity reason) {
    ReasonEntity oldReason = this.findByIdV10(id);
    //更新类型对象：设置updaterId和updatedAt
    reason.setId(id);
    BeanSolver.copyProperties(reason, oldReason);
    //保存原因主体
    return reasonRepository.save(oldReason);
  }
}
