package tv.acfun.cloud.service.limit.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RestController;
import tv.acfun.cloud.common.entity.Paginate;
import tv.acfun.cloud.common.mongo.SequenceRepository;
import tv.acfun.cloud.common.resolver.JsonArgument;
import tv.acfun.cloud.common.util.BeanSolver;
import tv.acfun.cloud.service.limit.LimitConstants;
import tv.acfun.cloud.service.limit.TypeService;
import tv.acfun.cloud.service.limit.entity.TypeEntity;
import tv.acfun.cloud.service.limit.repository.TypeRepository;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by wzp on 16/02/02.
 */
@RestController
@CacheConfig(cacheNames = LimitConstants.TYPES_CACHE)
public class TypeController implements TypeService {

  @Autowired
  private SequenceRepository sequenceRepository;
  @Autowired
  private TypeRepository typeRepository;

  @Transactional(readOnly = true)
  @Cacheable
  public Paginate findByV10(@JsonArgument TypeEntity type, @JsonArgument Paginate paginate) {
    if (paginate == null) {
      paginate = new Paginate();
    }
    //获得数据结果
    List<TypeEntity> result = new ArrayList<>();
    if (type == null || type.getCreaterId() == null) {
      result = typeRepository.findByDeletedAtNull(paginate.getPageRequest());
    } else {
      result = typeRepository.findByCreaterIdAndDeletedAtNull(type.getCreaterId(), paginate.getPageRequest());
    }
    return new Paginate(result);
  }

  @Transactional(readOnly = true)
  @Cacheable
  public TypeEntity findByIdV10(@PathVariable Long id) {
    return typeRepository.findByIdAndDeletedAtNull(id);
  }

  @Transactional
  @CacheEvict(allEntries = true, value = {LimitConstants.TYPES_CACHE, LimitConstants.LIMITS_CACHE})
  public TypeEntity deleteByIdV10(@PathVariable Long id, @JsonArgument TypeEntity type) {
    TypeEntity oldType = this.findByIdV10(id);
    //更新类型对象：设置deleterId和deletedAt
    type.setId(id);
    BeanSolver.copyProperties(type, oldType);
    return typeRepository.save(oldType);
  }

  @Transactional
  @CacheEvict(allEntries = true, value = {LimitConstants.TYPES_CACHE, LimitConstants.LIMITS_CACHE})
  public TypeEntity saveV10(@JsonArgument TypeEntity type) {
    //保存类型主体
    Long typeId = sequenceRepository.getNextValue(LimitConstants.TYPES_ID_KEY);
    type.setId(typeId);
    return typeRepository.save(type);
  }

  @Transactional
  @CacheEvict(allEntries = true, value = {LimitConstants.TYPES_CACHE, LimitConstants.LIMITS_CACHE})
  public TypeEntity updateByIdV10(@PathVariable Long id, @JsonArgument TypeEntity type) {
    TypeEntity oldType = this.findByIdV10(id);
    //更新类型对象：设置updaterId和updatedAt
    type.setId(id);
    BeanSolver.copyProperties(type, oldType);
    //保存类型主体
    return typeRepository.save(oldType);
  }
}
